# U-BOOT TCL File
#
# (C) Copyright 2007 Michal Simek
#
# Michal  SIMEK <monstr@monstr.eu>
#
# See file CREDITS for list of people who contributed to this
# project.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA
#
# ChangeLOG:
# 1.00.d clear Czech comments
# 1.00.c synchronization FLASH_MEMORY

# Globals variable
set drvlist [list]

proc uboot_drc {os_handle} {
    puts "\#--------------------------------------"
    puts "\# uboot BSP DRC...!"
    puts "\#--------------------------------------"
}


proc generate {os_handle} {
    variable drvlist
    puts "\#--------------------------------------"
    puts "\# uboot BSP generate..."
    puts "\#--------------------------------------"

# open config files
    set config_file_name "xparameters.h"
    set config_file [open $config_file_name w]

# print GPL license
    headerc $config_file
    puts $config_file ""

    set proc_handle [xget_libgen_proc_handle]
# ************************************************************************************************************
# print system clock
    set sys_clock [xget_sw_parameter_value $proc_handle "CORE_CLOCK_FREQ_HZ"]
    puts $config_file "/* System Clock Frequency */"
    puts $config_file "#define XILINX_CLOCK_FREQ\t$sys_clock"
    puts $config_file ""

# ************************************************************************************************************
# Microblaze
    set hwproc_handle [xget_handle $proc_handle "IPINST"]
    set args [xget_hw_parameter_handle $hwproc_handle "*"]
    set proctype [xget_value $hwproc_handle "OPTION" "IPNAME"]
    switch $proctype {
        "microblaze" {
# write only name of instance
 	  foreach arg $args {
		set arg_name [xget_value $arg "NAME"]
		set arg_name [string map -nocase {C_ ""} $arg_name]
		set arg_value [xget_value $arg "VALUE"]
#		puts "DEBUG $arg_name $arg_value"
		switch -regexp $arg_name {
			INSTANCE {
				   	puts $config_file "/* Microblaze is $arg_value */"
					break;
			}
			default {}
		}
           }
 	  foreach arg $args {
		set arg_name [xget_value $arg "NAME"]
		set arg_name [string map -nocase {C_ ""} $arg_name]
		set arg_value [xget_value $arg "VALUE"]
#		puts "DEBUG $arg_name $arg_value"
		switch -regexp $arg_name {
			FSL_LINKS {
				   	puts $config_file "#define XILINX_FSL_NUMBER\t$arg_value"
			}
			default {}
		}
           }
        }
    }
    puts $config_file ""
# ************************************************************************************************************
# Interrupt controler
    set intc [xget_sw_parameter_value $os_handle "intc"]
    if {[string match "" $intc] || [string match -nocase "none" $intc]} {
            puts $config_file "/* Interrupt controller not defined */"
    } else {
	    puts $config_file "/* Interrupt controller is $intc */"
# Finding all peripherals for intc drivers - FIXME - I think that the periphery is only one
	    set per_num [xget_handle $proc_handle "DRIVER" "intc"]
	    set per_mhs_param [xget_sw_iplist_for_driver $per_num]
# Interrupt controller address
	    set baseaddr [xget_sw_parameter_handle $per_mhs_param "C_BASEADDR"]
	    set baseaddr_value [xget_value $baseaddr "VALUE"]
            set baseaddr_value [format "0x%08x" $baseaddr_value]
	    puts $config_file "#define XILINX_INTC_BASEADDR\t$baseaddr_value"
	    set number_intr [xget_sw_parameter_handle $per_mhs_param "C_NUM_INTR_INPUTS"]
	    set intr_value [xget_value $number_intr "VALUE"]
	    puts $config_file "#define XILINX_INTC_NUM_INTR_INPUTS\t$intr_value"
	    set intr_value [expr $intr_value - 1]
	    set intc_handle [xget_sw_ipinst_handle_from_processor $proc_handle $intc]
    	    set port_list [xget_port_by_subtype $intc_handle "*"]
    	    foreach port $port_list {
    	     	    set name [xget_value $port "NAME"]
    	     	    if {[string match -nocase $name "intr"]} {
		       	    set intr_irq [xget_value $port "VALUE"]
#		       	    puts $config_file "/* pripojene interrupty $name=$intr_irq */"
    	     	    }
    	    }
     }
    puts $config_file ""
# ************************************************************************************************************
# Timer part
    set timer [xget_sw_parameter_value $os_handle "timer"]
    if {[string match "" $timer] || [string match -nocase "none" $timer]} {
            puts $config_file "/* Timer not defined */"
    } else {
	    set timer_handle [xget_sw_ipinst_handle_from_processor $proc_handle $timer]
	    set timer_start [xget_sw_parameter_value $timer_handle "C_BASEADDR"]
	    set timer_start [format "0x%08x" $timer_start]
	    puts $config_file "/* Timer pheriphery is $timer */"
	    puts $config_file "#define XILINX_TIMER_BASEADDR\t$timer_start"

	    if {![string match "" $intc] && ![string match -nocase "none" $intc]} {
		    set port_list [xget_port_by_subtype $timer_handle "*"]
		    foreach port $port_list {
		     	    set name [xget_value $port "NAME"]
		     	    if {[string match -nocase $name "interrupt"]} {
		  	     	    set value [xget_value $port "VALUE"]
# [CZECH] spocitam kolikata polozka je interrupt pro timer - max delka
			    	    set delka $intr_value
# [CZECH] cyklus pres vsechny pripojene draty
				    foreach cislo $intr_irq {
# [CZECH] mimo konkatenace - tu preskoc
				    	    if {![string match $cislo "&"]} {
# [CZECH] pokud jsi nasel stejne cislo dratu tak skonci
					     	    if {[string match -nocase $cislo $value]} {
					     	        puts $config_file "#define XILINX_TIMER_IRQ\t$delka"
					     	    }
# [CZECH] odcitej do nuly
					     	    set delka [expr $delka - 1]
			 	    	    }
				    }
		     	    }
		    }
	     }


    }
    puts $config_file ""
# ************************************************************************************************************
# UartLite driver - I suppose, only uartlite driver
    set uart [xget_sw_parameter_value $os_handle "stdin_out"]
    if {[string match "" $uart] || [string match -nocase "none" $uart]} {
            puts $config_file "/* Uartlite not defined */"
            error "ERROR Uartlite not specified. Please specific console"
    } else {
	    set uart_handle [xget_sw_ipinst_handle_from_processor $proc_handle $uart]
	    set uart_baseaddr [xget_sw_parameter_value $uart_handle "C_BASEADDR"]
	    set uart_baseaddr [format "0x%08x" $uart_baseaddr]
	    set uart_baudrate [xget_sw_parameter_value $uart_handle "C_BAUDRATE"]
	    puts $config_file "/* Uart pheriphery is $uart */"
	    puts $config_file "#define XILINX_UART_BASEADDR\t$uart_baseaddr"
	    puts $config_file "#define XILINX_UART_BAUDRATE\t$uart_baudrate"
    }
    puts $config_file ""
# ************************************************************************************************************
# IIC driver - I suppose, only uartlite driver
    set iic [xget_sw_parameter_value $os_handle "iic"]
    if {[string match "" $iic] || [string match -nocase "none" $iic]} {
            puts $config_file "/* IIC doesn't exist */"
    } else {
	    set iic_handle [xget_sw_ipinst_handle_from_processor $proc_handle $iic]
	    set iic_baseaddr [xget_sw_parameter_value $iic_handle "C_BASEADDR"]
	    set iic_baseaddr [format "0x%08x" $iic_baseaddr]
	    set iic_freq [xget_sw_parameter_value $iic_handle "C_IIC_FREQ"]
	    set iic_bit [xget_sw_parameter_value $iic_handle "C_TEN_BIT_ADR"]
	    puts $config_file "/* IIC pheriphery is $iic */"
	    puts $config_file "#define XILINX_IIC_0_BASEADDR\t$iic_baseaddr"
	    puts $config_file "#define XILINX_IIC_0_FREQ\t$iic_freq"
	    puts $config_file "#define XILINX_IIC_0_BIT\t$iic_bit"
    }
    puts $config_file ""
# ************************************************************************************************************
# GPIO configuration
    set gpio [xget_sw_parameter_value $os_handle "gpio"]
    if {[string match "" $gpio] || [string match "none" $gpio]} {
      	 puts $config_file "/* GPIO doesn't exist */"
    } else {
         set base_param_name [format "C_BASEADDR" $gpio]
	 set gpio_handle [xget_sw_ipinst_handle_from_processor $proc_handle $gpio]
	 set gpio_start [xget_sw_parameter_value $gpio_handle $base_param_name]
	 set gpio_start [format "0x%08x" $gpio_start]
	 puts $config_file "/* GPIO is $gpio*/"
	 puts $config_file "#define XILINX_GPIO_BASEADDR\t$gpio_start"
    }
    puts $config_file ""
# ************************************************************************************************************
# Flash memory
  	set flash_mem [xget_sw_parameter_value $os_handle "flash_memory"]
     if {[string match "" $flash_mem] || [string match "none" $flash_mem]} {
      	 puts $config_file "/* FLASH doesn't exist $flash_mem */"
      	 puts "FLASH doesn't exists"
     } else {
	set flash_mem_handle [xget_sw_ipinst_handle_from_processor $proc_handle $flash_mem]
        set flash_mem_bank [xget_sw_parameter_value $os_handle "flash_memory_bank"]
	set base_param_name [format "C_MEM%i_BASEADDR" $flash_mem_bank]
	set high_param_name [format "C_MEM%i_HIGHADDR" $flash_mem_bank]
	set flash_start [xget_sw_parameter_value $flash_mem_handle $base_param_name]
	set flash_end [xget_sw_parameter_value $flash_mem_handle $high_param_name]
	set flash_size [expr $flash_end - $flash_start + 1]
	set flashm_start [format "0x%08x" $flash_start]
	set flash_size [format "0x%08x" $flash_size]
	puts $config_file "/* Flash Memory is $flash_mem */"
	puts $config_file "#define XILINX_FLASH_START\t$flash_start"
	puts $config_file "#define XILINX_FLASH_SIZE\t$flash_size"
    }
    puts $config_file ""
# ************************************************************************************************************
# System memory
    set main_mem [xget_sw_parameter_value $os_handle "main_memory"]
    if {[string match "" $main_mem] || [string match "none" $main_mem]} {
         error "ERROR main_memory not specified. Please specific main_memory"
    } else {
	    set main_mem_bank [xget_sw_parameter_value $os_handle "main_memory_bank"]
	    set main_mem_handle [xget_sw_ipinst_handle_from_processor $proc_handle $main_mem]
	    if {[string compare -nocase $main_mem_handle ""] != 0} {
	        set base_param_name [format "C_MEM%i_BASEADDR" $main_mem_bank]
	        set high_param_name [format "C_MEM%i_HIGHADDR" $main_mem_bank]
		set eram_base [xget_sw_parameter_value $main_mem_handle $base_param_name]
		set eram_end [xget_sw_parameter_value $main_mem_handle $high_param_name]
		set eram_size [expr $eram_end - $eram_base + 1]
		set eram_base [format "0x%08x" $eram_base]
		set eram_size [format "0x%08x" $eram_size]
		set eram_high [expr $eram_base + $eram_size]
		set eram_high [format "0x%08x" $eram_high]
		puts $config_file "/* Main Memory is $main_mem */"
		puts $config_file "#define XILINX_RAM_START\t$eram_base"
		puts $config_file "#define XILINX_RAM_SIZE\t$eram_size"
	    }
    }
    puts $config_file ""
# ************************************************************************************************************
# Sysace
     set sysace [xget_sw_parameter_value $os_handle "sysace"]
     if {[string match "" $sysace] || [string match "none" $sysace]} {
      	 puts $config_file "/* Sysace doesn't exist */"
     } else {
	    puts $config_file "/* Sysace Controller is $sysace */"
	    set drv [xget_handle $proc_handle "DRIVER" "sysace"]
# device_id is fixed on value
	    set periphs [xget_sw_iplist_for_driver $drv]
	    foreach periph $periphs {
# get sw parameters
	    	    set args [xget_sw_parameter_handle $periph "*"]
	    	    foreach arg $args {
			set arg_name [xget_value $arg "NAME"]
			set arg_name [string map -nocase {C_ ""} $arg_name]
#		        puts $config_file "$arg_name $arg_value"
# case example and usage
			case $arg_name in {
			    {"BASEADDR" "HIGHADDR" "MEM_WIDTH"} {
			    		set arg_value [xget_value $arg "VALUE"]
					puts $config_file "#define XILINX_SYSACE_${arg_name}\t$arg_value"
					}
			    default	{}
			}
	    	    }
	    }
    }
    puts $config_file ""
# ************************************************************************************************************
# Ethernet
     set ethernet [xget_sw_parameter_value $os_handle "ethernet"]
     if {[string match "" $ethernet] || [string match "none" $ethernet]} {
      	 puts $config_file "/* ethernet doesn't exist */"
     } else {
	    puts $config_file "/* Ethernet controller is $ethernet */"
	    set drv [xget_handle $proc_handle "DRIVER" "emac"]
	    set pocet [llength $drv]
	    puts $config_file "#define XPAR_XEMAC_NUM_INSTANCES\t$pocet"
# device_id is fixed on value
	    puts $config_file "#define XPAR_OPB_ETHERNET_0_DEVICE_ID\t[expr $pocet - 1]"
	    set periphs [xget_sw_iplist_for_driver $drv]
	    foreach periph $periphs {
# getting sw parameters
	    	    set args [xget_sw_parameter_handle $periph "*"]
	    	    foreach arg $args {
			set arg_name [xget_value $arg "NAME"]
			set arg_name [string map -nocase {C_ ""} $arg_name]
#		        puts $config_file "$arg_name $arg_value"
# case example
			case $arg_name in {
			    {"BASEADDR" "HIGHADDR" "ERR_COUNT_EXIST" "DMA_PRESENT" "MII_EXIST"} {
			    		set arg_value [xget_value $arg "VALUE"]
					puts $config_file "#define XPAR_OPB_ETHERNET_0_${arg_name}\t$arg_value"
					}
			    default	{}
			}
	    	    }
	    }
    }
#    puts $config_file ""
# close the file...
    close $config_file
#*************************************************************************************************************
# Open config.mk file
    set config_file_name "config.mk"
    set config_file [open $config_file_name w]

# Print GPL
    headerm $config_file
    puts $config_file ""

# U-BOOT position in memory
    set text_base [xget_sw_parameter_value $os_handle "uboot_position"]
    set text_base [format "0x%08x" $text_base]
    if {$eram_base < $text_base && $eram_high > $text_base} {
#	    puts $config_file "# TEXT BASE "
	    puts $config_file "TEXT_BASE = $text_base"
	    puts $config_file ""
# FIXME static parameters
	    puts $config_file "PLATFORM_CPPFLAGS += -mno-xl-soft-mul"
	    puts $config_file "PLATFORM_CPPFLAGS += -mno-xl-soft-div"
	    puts $config_file "PLATFORM_CPPFLAGS += -mxl-barrel-shift"

# print system clock
#    set sw [xget_sw_parameter_value $proc_handle "HW_INSTANCE"]

# FIXME Parameters for Microblaze from MHS files - look at Microblaze SW manual
# print microblaze params
    set hwproc_handle [xget_handle $proc_handle "IPINST"]
    set args [xget_hw_parameter_handle $hwproc_handle "*"]
    set proctype [xget_value $hwproc_handle "OPTION" "IPNAME"]
    switch $proctype {
        "microblaze" {
 	  foreach arg $args {
		set arg_name [xget_value $arg "NAME"]
		set arg_name [string map -nocase {C_ ""} $arg_name]
		set arg_value [xget_value $arg "VALUE"]
#		puts "DEBUG $arg_name $arg_value"
		switch -regexp $arg_name {
			USE_BARREL {
				if {[string match $arg_value "1"]} {
#				   	puts $config_file "PLATFORM_CPPFLAGS += -mxl-barrel-shift"
				}
			}
			USE_DIV {
				if {[string match $arg_value "1"]} {
#				   	puts $config_file "PLATFORM_CPPFLAGS += -mno-xl-hard-div"
				} else {
#					puts $config_file "PLATFORM_CPPFLAGS += -mno-xl-soft-div"
				}
			}
			USE_HW_MULT {
				if {[string match $arg_value "1"]} {
#				   	puts $config_file "PLATFORM_CPPFLAGS += -mxl-soft-mul"
				} else {
#					puts $config_file "PLATFORM_CPPFLAGS += -mno-xl-soft-mul"
				}
			}
			default {}
		}
           }
        }
        "ppc405" -
       "ppc405_virtex4" {
            puts "unsupported processor type $proctype\n"
            error "ERROR $proc_type not supported"
        }
        default {
            puts "unsupported processor type $proctype\n"
            error "ERROR $proc_type not supported"
        }
    }
    } else {
      	    error "ERROR u-boot position is out of range $eram_base - $eram_high"
    }

#    puts $config_file ""
    close $config_file
#*************************************************************************************************************
}

##################################################
# procedure post_generate
# This generates the drivers directory for uboot
# and runs the ltypes script
##################################################

proc post_generate {lib_handle} {
    set pwd [pwd]
    set name [xget_value $lib_handle "PARAMETER" name]
    if {$name != ""} {
	    set uboot_dir "uboot/board/xilinx/$name"
	    exec bash -c "mkdir -p $uboot_dir"
	    exec bash -c "cp ../../include/xparameters.h $uboot_dir/xparameters-edk.h"
	    exec bash -c "cp ../../../system.mhs $uboot_dir"
	    exec bash -c "cp ../../../system.mss $uboot_dir"
	    exec bash -c "mv xparameters.h $uboot_dir"
	    exec bash -c "mv config.mk $uboot_dir"
    }
}

proc headerm {config_file} {
     puts $config_file "\#"
     puts $config_file "\# (C) Copyright 2007 Michal Simek"
     puts $config_file "\#"
     puts $config_file "\# Michal  SIMEK <monstr@monstr.eu>"
     puts $config_file "\#"
     puts $config_file "\# See file CREDITS for list of people who contributed to this"
     puts $config_file "\# project."
     puts $config_file "\#"
     puts $config_file "\# This program is free software; you can redistribute it and/or"
     puts $config_file "\# modify it under the terms of the GNU General Public License as"
     puts $config_file "\# published by the Free Software Foundation; either version 2 of"
     puts $config_file "\# the License, or (at your option) any later version."
     puts $config_file "\#"
     puts $config_file "\# This program is distributed in the hope that it will be useful,"
     puts $config_file "\# but WITHOUT ANY WARRANTY; without even the implied warranty of"
     puts $config_file "\# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the"
     puts $config_file "\# GNU General Public License for more details."
     puts $config_file "\#"
     puts $config_file "\# You should have received a copy of the GNU General Public License"
     puts $config_file "\# along with this program; if not, write to the Free Software"
     puts $config_file "\# Foundation, Inc., 59 Temple Place, Suite 330, Boston,"
     puts $config_file "\# MA 02111-1307 USA"
     puts $config_file "\#"
     puts $config_file "\# CAUTION: This file is automatically generated by libgen."
     puts $config_file "\# Version: [xget_swverandbld]"
     puts $config_file "\#"
}

proc headerc {config_file} {
     puts $config_file "/*"
     puts $config_file " * (C) Copyright 2007 Michal Simek"
     puts $config_file " *"
     puts $config_file " * Michal  SIMEK <monstr@monstr.eu>"
     puts $config_file " *"
     puts $config_file " * See file CREDITS for list of people who contributed to this"
     puts $config_file " * project."
     puts $config_file " *"
     puts $config_file " * This program is free software; you can redistribute it and/or"
     puts $config_file " * modify it under the terms of the GNU General Public License as"
     puts $config_file " * published by the Free Software Foundation; either version 2 of"
     puts $config_file " * the License, or (at your option) any later version."
     puts $config_file " *"
     puts $config_file " * This program is distributed in the hope that it will be useful,"
     puts $config_file " * but WITHOUT ANY WARRANTY; without even the implied warranty of"
     puts $config_file " * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the"
     puts $config_file " * GNU General Public License for more details."
     puts $config_file " *"
     puts $config_file " * You should have received a copy of the GNU General Public License"
     puts $config_file " * along with this program; if not, write to the Free Software"
     puts $config_file " * Foundation, Inc., 59 Temple Place, Suite 330, Boston,"
     puts $config_file " * MA 02111-1307 USA"
     puts $config_file " *"
     puts $config_file " * CAUTION: This file is automatically generated by libgen."
     puts $config_file " * Version: [xget_swverandbld]"
     puts $config_file " */"
}

